/*
###########################################################################
# trustyRC, fully modular IRC robot 
# Copyright (C) 2006-2008 Nicoleau Fabien
#
# trustyRC is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 0.1.2-1307 USA
###########################################################################
*/

/** @file main.cpp
 * @brief Main program.
 *
 * Construct a botkernel and launch it.
 */

#include <unistd.h>
#include "botkernel.h"

void displayHelp(string,bool);

int main(int nbArgs, char *arrayArgs[]) 
{ 
   char * pHOME = NULL;
   struct stat st;
   srand((unsigned)time(NULL));
   pid_t pid;
   string confFile = "" ;
	char optchar;
	bool background=false;
	opterr = 0 ;	
   pHOME = getenv("HOME");
   if ( pHOME != NULL ) {
      confFile = (string)pHOME+"/.trustyrc.conf";
      if(stat(confFile.c_str(),&st) == -1) {
         confFile = "/etc/trustyrc.conf";
      }
   }
   else {
      confFile = "/etc/trustyrc.conf";
   }
	while ( (optchar = getopt(nbArgs,arrayArgs,"-bc:")) != EOF ) {
		switch (optchar) {
			case 'c' : confFile = optarg;
			break;
			case 'b' : background = true;
			break;
			case 1 : cout << "Invalid argument : "<<  optarg  << endl;
                  displayHelp(arrayArgs[0],true);
			break;
			default: 
			case '?' : cout << "Invalid option : "<< (char) optopt << endl;
                    displayHelp(arrayArgs[0],true);
			break;
		}	
	}
	BotKernel bot(confFile);
   if (background) {
      pid =  fork();
      switch (pid) {
         case -1 : cout << "Fork ERROR !, exiting" << endl;
         exit(-1);
         break;
         case 0 : // son
            setsid();
            bot.run();
         break;
         default :  // father 
            exit(0);
      }
   }
   else {
      bot.run();
   }
   return 0;
}

void displayHelp(string firstArg,bool quit)
{
   cout << "Usage : "<< firstArg << " [-c configuration_file] [-b]" << endl;
   if ( quit ) {
      exit(0);
   }
}
