/*
###########################################################################
# trustyRC, fully modular IRC robot 
# Copyright (C) 2006-2008 Nicoleau Fabien
#
# trustyRC is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 0.1.2-1307 USA
###########################################################################
*/

/** @file module.cpp
 * @brief Module implementation file
 */

#include "module.h"

/**
 * Constructor
 */ 
Module::Module(BotKernel*b)
{
	this->author = "eponyme";
	this->description = "bot modules management";
	this->version = "0.1.2";
	this->name = "module";
	this->bindFunction("load",IN_COMMAND_HANDLER,"load",0,10);
	this->bindFunction("unload",IN_COMMAND_HANDLER,"unload",0,10);
	this->bindFunction("listmodules",IN_COMMAND_HANDLER,"listmodules",0,10);
	this->bindFunction("listlibs",IN_COMMAND_HANDLER,"listlibs",0,10);
	this->bindFunction("moduleinfos",IN_COMMAND_HANDLER,"moduleinfos",0,10);
}

extern "C"
{
	Plugin *contruct_module(BotKernel*b)
	{
		return new Module(b);
	}
	void destroy_module(Plugin*p)
	{
		delete p;
	}
   bool load (Message*m,Plugin*p,BotKernel*b)
   {
      bool exec = false;
      string buffer;
      pPlugin * pp = b->getPlugin("admin");
      Admin * adm ;
      if ( pp != NULL )
      {
         adm = (Admin*) pp->object;
      }
      else 
      {
         adm = NULL;  
      }    
      if(m->isPrivate())
      {
         if ( m->getSplit().size() == 5 )
         {
            if ( adm == NULL )
            {
               exec = true;
            }
            else
            {
               if (adm->isSuperAdmin( m->getSender()))
               { 
                  exec = true;
               }
               else
               {
                  exec = false;
               }
            }
            if ( exec )
            {
               if(b->loadPlugin(m->getPart(4)))
               {
                  b->send(IRCProtocol::sendNotice(m->getNickSender(),m->getPart(4)+" loaded"));
                  b->getSysLog()->log(INFO,m->getPart(4)+" loaded by "+m->getSender());
               }
               else
               {
                  b->send(IRCProtocol::sendNotice(m->getNickSender(),m->getPart(4)+" loading failed."));
                  b->getSysLog()->log(WARNING,m->getPart(4)+" loading failed (by "+m->getSender()+")");
               }
            }
         }
      }
		return true;
   }
   bool unload (Message*m,Plugin*p,BotKernel*b)
   {
      bool exec = false;
      string buffer;
      pPlugin * pp = b->getPlugin("admin");
      Admin * adm ;
      if ( pp != NULL ) {
         adm = (Admin*) pp->object;
      }
      else  {
         adm = NULL;  
      }   
      if(m->isPrivate()) {
         if ( m->getSplit().size() == 5 )  {
            if (m->getPart(4) == p->getName()) { // Protection to not unload "module" plugin
               return true;
            }
            if ( adm == NULL ) {
               exec = true;
            }
            else {
               if (adm->isSuperAdmin( m->getSender())) { 
                  exec = true;
               }
               else {
                  exec = false;
               }
            }
            if ( exec ) {
               if(b->unloadPlugin(m->getPart(4))) {
                  b->send(IRCProtocol::sendNotice(m->getNickSender(),m->getPart(4)+" unloaded"));
                  b->getSysLog()->log(INFO,m->getPart(4)+" unloaded by "+m->getSender());
               }
               else {
                  b->send(IRCProtocol::sendNotice(m->getNickSender(),m->getPart(4)+" unloading failed."));
                  b->getSysLog()->log(WARNING,m->getPart(4)+" unloading failed (by "+m->getSender()+")");
               }
            }
         }
      }
		return true;
   }
   bool listmodules (Message*m,Plugin*p,BotKernel*b)
   {
      bool exec = false;
      string buffer;
      pPlugin * pp = b->getPlugin("admin");
      Admin * adm ;
      if ( pp != NULL )
      {
         adm = (Admin*) pp->object;
      }
      else 
      {
         adm = NULL;  
      }     
      if(m->isPrivate())
      {
         if ( adm == NULL )
         {
            exec = true;
         }
         else
         {
            if (adm->isSuperAdmin( m->getSender()))
            { 
               exec = true;
            }
            else
            {
               exec = false;
            }
         }
         if ( exec )
         {
             b->send(IRCProtocol::sendNotices(m->getNickSender(),Tools::gatherVectorElements(b->getPluginsList()," ",4)));
         }
      }
		return true;
   }
   bool listlibs (Message*m,Plugin*p,BotKernel*b) {
      DIR *dp;
      struct dirent *ep;
      vector<string> plugins;
      pPlugin * ppAdm = b->getPlugin("admin");
      Admin*adm = NULL;
      if ((ppAdm!=NULL) && m->isPrivate() ) {
         adm = (Admin*)ppAdm->object ;
         if (adm->isSuperAdmin(m->getSender()) ) {
            dp = opendir (b->getCONFF()->getValue("kernel.plugdir").c_str());
            if (dp != NULL) {
               while ((ep = readdir(dp))) {
                  if ( ((string)ep->d_name != ".") && ((string)ep->d_name !="..")) {
                     plugins.push_back(ep->d_name);
                  }
               }
               closedir (dp);
            }
            else {
               plugins.push_back("* Couldn't open the directory *");
            }
            b->send(IRCProtocol::sendNotices(m->getNickSender(),Tools::gatherVectorElements(plugins," ",4)));
         }
      }
      return true;
   }
   bool moduleinfos (Message*m,Plugin*p,BotKernel*b) {
      pPlugin * pp = NULL;
      pPlugin * ppAdm = b->getPlugin("admin");
      Admin*adm = NULL;
      Plugin * plug;
      if ((ppAdm!=NULL) && m->isPrivate() && (m->nbParts() == 5)) {
         adm = (Admin*)ppAdm->object ;
         if (adm->isSuperAdmin(m->getSender()) ) {
            pp = b->getPlugin(m->getPart(4) ) ;
            if ( pp != NULL ) {
               plug = pp->object;
               b->send(IRCProtocol::sendNotice(m->getNickSender(),m->getPart(4) +" ("+plug->getVersion()+") by "+plug->getAuthor()+" : "+plug->getDescription()));
            }
            else {
               b->send(IRCProtocol::sendNotice(m->getNickSender(),m->getPart(4) +" not loaded"));
            }
         }
      }
      return true;
   }
}
