/*
###########################################################################
# trustyRC, fully modular IRC robot 
# Copyright (C) 2006-2008 Nicoleau Fabien
#
# trustyRC is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 0.1.2-1307 USA
###########################################################################
*/

/** @file usersinfos.h
 * @brief UsersInfos header file
 */

#ifndef USERSINFOS_H
#define USERSINFOS_H
#include "../plugin.h"
#include "../botkernel.h"
#include "../channel.h"
#include <string>
#include <map>
using namespace std;

/** @brief Follow users modes on channels
 *
 * This plugin gives functions to follow users modes
 * on channels and keep informations (nick,ident,host,status)
 * Can be usefull for operator commands and ban masks
 */

class UsersInfos : public Plugin
{
private:
	/// Vector for prefixes translation (@=>o,+=>v)
	vector<string> prefixes;
	/// Channels storage
	map<string,Channel*> users;
public:
   /// Constructor
	UsersInfos(BotKernel*);
	/// Destructor
	~UsersInfos();
	/// Add a prefixe translation
	void addPrefixe(char,char);
	/// Get a prefixe translation
	char getPrefixe(char);
	/// Return the users attribute pointer
	map<string,Channel*> * getUsers();
   /// tell if a user has the given mode on a given channel
   bool hasMode(string,string,char);
};


#endif

