/*
###########################################################################
# trustyRC, fully modular IRC robot 
# Copyright (C) 2006-2008 Nicoleau Fabien
#
# trustyRC is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 0.1.2-1307 USA
###########################################################################
*/

/** @file syslog.h
 * @brief SysLog header file
 */

#ifndef SYSLOG_H
#define SYSLOG_H

#include <sys/stat.h>
#include <string>
#include <iostream>
#include <fstream>
using namespace std;

/// Log levels
typedef enum {NONE,ERROR,WARNING,INFO} log_level;

/** @brief Class that manage log system
 *
 * This class is used to log bot events.
 * Differents levels are available :
 * error, warning, info and none
 */

class SysLog {
	private:
		/// log level used
		log_level ll;
		/// If true, display events
		bool verbose;
		/// if different from "0", the logfile is backed up instead of erased when its size is bigger thant maxSize
		string backup;
		/// Log file (path and name)
		string file;
		/// Log file max size (in mega)
		int maxSize ;
      /// File stream
      ofstream* myStream;
	public:
		/// Constructor
		SysLog(log_level,bool,int,string,string);
		/// Constructor
		SysLog(string,bool,int,string,string);
		/// Destructor
		~SysLog();
		/// Set the log level
		void setLogLevel(log_level);
		/// Set the log level
		void setLogLevel(string);
		/// Set if the bot must display events
		void setVerbose(bool);
		/// Set log file path+name
		void setFile(string);
		/// log an event
		void log(log_level,string);
		/// Set max backup file size
		void setMaxSize(int);
		/// Set if the log file must be backed up when is size is bigger than maxSize
		void setBackup(string);
      /// Open the log file
      bool open() ;
      /// Close the log file
      bool close() ;
};

#endif
